@@ -43,7 +43,8 @@ type Server struct {
 }
 
 func NewGrpcServer(b *server.BgpServer, hosts string) *Server {
-	return NewServer(b, grpc.NewServer(), hosts)
+	size := 256 << 20
+	return NewServer(b, grpc.NewServer(grpc.MaxRecvMsgSize(size), grpc.MaxSendMsgSize(size)), hosts)
 }
 
 func NewServer(b *server.BgpServer, g *grpc.Server, hosts string) *Server {
@@ -97,10 +98,10 @@ func NewMpGracefulRestartFromConfigStruct(c *config.MpGracefulRestart) *MpGracef
 	}
 }
 
-func NewAfiSafiConfigFromConfigStruct(c *config.AfiSafiConfig) *AfiSafiConfig {
+func NewAfiSafiConfigFromConfigStruct(c *config.AfiSafi) *AfiSafiConfig {
 	return &AfiSafiConfig{
-		Family:  uint32(bgp.AddressFamilyValueMap[string(c.AfiSafiName)]),
-		Enabled: c.Enabled,
+		Family:  uint32(c.State.Family),
+		Enabled: c.Config.Enabled,
 	}
 }
 
@@ -166,13 +167,12 @@ func NewUseMultiplePathsFromConfigStruct(c *config.UseMultiplePaths) *UseMultipl
 }
 
 func NewPrefixLimitFromConfigStruct(c *config.AfiSafi) *PrefixLimit {
-	family, err := bgp.GetRouteFamily(string(c.Config.AfiSafiName))
-	if err != nil || c.PrefixLimit.Config.MaxPrefixes == 0 {
+	if c.PrefixLimit.Config.MaxPrefixes == 0 {
 		return nil
 	}
 
 	return &PrefixLimit{
-		Family:               uint32(family),
+		Family:               uint32(c.State.Family),
 		MaxPrefixes:          c.PrefixLimit.Config.MaxPrefixes,
 		ShutdownThresholdPct: uint32(c.PrefixLimit.Config.ShutdownThresholdPct),
 	}
@@ -198,7 +198,7 @@ func NewLongLivedGracefulRestartFromConfigStruct(c *config.LongLivedGracefulRest
 func NewAfiSafiFromConfigStruct(c *config.AfiSafi) *AfiSafi {
 	return &AfiSafi{
 		MpGracefulRestart:        NewMpGracefulRestartFromConfigStruct(&c.MpGracefulRestart),
-		Config:                   NewAfiSafiConfigFromConfigStruct(&c.Config),
+		Config:                   NewAfiSafiConfigFromConfigStruct(c),
 		ApplyPolicy:              NewApplyPolicyFromConfigStruct(&c.ApplyPolicy),
 		RouteSelectionOptions:    NewRouteSelectionOptionsFromConfigStruct(&c.RouteSelectionOptions),
 		UseMultiplePaths:         NewUseMultiplePathsFromConfigStruct(&c.UseMultiplePaths),
@@ -213,9 +213,7 @@ func NewPeerFromConfigStruct(pconf *config.Neighbor) *Peer {
 	prefixLimits := make([]*PrefixLimit, 0, len(pconf.AfiSafis))
 	afiSafis := make([]*AfiSafi, 0, len(pconf.AfiSafis))
 	for _, f := range pconf.AfiSafis {
-		if family, ok := bgp.AddressFamilyValueMap[string(f.Config.AfiSafiName)]; ok {
-			families = append(families, uint32(family))
-		}
+		families = append(families, uint32(f.State.Family))
 		if prefixLimit := NewPrefixLimitFromConfigStruct(&f); prefixLimit != nil {
 			prefixLimits = append(prefixLimits, prefixLimit)
 		}
@@ -468,6 +466,58 @@ func (s *Server) GetRib(ctx context.Context, arg *GetRibRequest) (*GetRibRespons
 	}, err
 }
 
+func (s *Server) GetPath(arg *GetPathRequest, stream GobgpApi_GetPathServer) error {
+	f := func() []*table.LookupPrefix {
+		l := make([]*table.LookupPrefix, 0, len(arg.Prefixes))
+		for _, p := range arg.Prefixes {
+			l = append(l, &table.LookupPrefix{
+				Prefix:       p.Prefix,
+				LookupOption: table.LookupOption(p.LookupOption),
+			})
+		}
+		return l
+	}
+
+	in := false
+	family := bgp.RouteFamily(arg.Family)
+	var tbl *table.Table
+	var err error
+	switch arg.Type {
+	case Resource_LOCAL, Resource_GLOBAL:
+		tbl, err = s.bgpServer.GetRib(arg.Name, family, f())
+	case Resource_ADJ_IN:
+		in = true
+		fallthrough
+	case Resource_ADJ_OUT:
+		tbl, err = s.bgpServer.GetAdjRib(arg.Name, family, in, f())
+	case Resource_VRF:
+		tbl, err = s.bgpServer.GetVrfRib(arg.Name, family, []*table.LookupPrefix{})
+	default:
+		return fmt.Errorf("unsupported resource type: %v", arg.Type)
+	}
+	if err != nil {
+		return err
+	}
+
+	return func() error {
+		for _, dst := range tbl.GetDestinations() {
+			for idx, path := range dst.GetAllKnownPathList() {
+				p := ToPathApi(path)
+				if idx == 0 {
+					switch arg.Type {
+					case Resource_LOCAL, Resource_GLOBAL:
+						p.Best = true
+					}
+				}
+				if err := stream.Send(p); err != nil {
+					return err
+				}
+			}
+		}
+		return nil
+	}()
+}
+
 func (s *Server) MonitorRib(arg *MonitorRibRequest, stream GobgpApi_MonitorRibServer) error {
 	if arg == nil || arg.Table == nil {
 		return fmt.Errorf("invalid request")
