@@ -140,21 +140,28 @@ func NewPeerInfo(g *config.Global, p *config.Neighbor) *PeerInfo {
 }
 
 type Destination struct {
-	routeFamily   bgp.RouteFamily
-	nlri          bgp.AddrPrefixInterface
-	knownPathList paths
-	withdrawList  paths
-	newPathList   paths
-	RadixKey      string
+	routeFamily      bgp.RouteFamily
+	nlri             bgp.AddrPrefixInterface
+	knownPathList    paths
+	withdrawList     paths
+	newPathList      paths
+	oldKnownPathList paths
+	RadixKey         string
+	localIdMap       *Bitmap
 }
 
-func NewDestination(nlri bgp.AddrPrefixInterface, known ...*Path) *Destination {
+func NewDestination(nlri bgp.AddrPrefixInterface, mapSize int, known ...*Path) *Destination {
 	d := &Destination{
 		routeFamily:   bgp.AfiSafiToRouteFamily(nlri.AFI(), nlri.SAFI()),
 		nlri:          nlri,
 		knownPathList: known,
 		withdrawList:  make([]*Path, 0),
 		newPathList:   make([]*Path, 0),
+		localIdMap:    NewBitmap(mapSize),
+	}
+	// the id zero means id is not allocated yet.
+	if mapSize != 0 {
+		d.localIdMap.Flag(0)
 	}
 	switch d.routeFamily {
 	case bgp.RF_IPv4_UC, bgp.RF_IPv6_UC, bgp.RF_IPv4_MPLS, bgp.RF_IPv6_MPLS:
@@ -226,50 +233,21 @@ func (dd *Destination) GetMultiBestPath(id string) []*Path {
 	return getMultiBestPath(id, &dd.knownPathList)
 }
 
-func (dd *Destination) AddWithdraw(withdraw *Path) {
-	dd.validatePath(withdraw)
-	dd.withdrawList = append(dd.withdrawList, withdraw)
-}
-
-func (dd *Destination) AddNewPath(newPath *Path) {
-	dd.validatePath(newPath)
-	dd.newPathList = append(dd.newPathList, newPath)
-}
-
-func (dd *Destination) validatePath(path *Path) {
-	if path == nil || path.GetRouteFamily() != dd.routeFamily {
-
-		log.WithFields(log.Fields{
-			"Topic":      "Table",
-			"Key":        dd.GetNlri().String(),
-			"Path":       path,
-			"ExpectedRF": dd.routeFamily,
-		}).Error("path is nil or invalid route family")
+func (dd *Destination) GetAddPathChanges(id string) []*Path {
+	l := make([]*Path, 0, len(dd.newPathList)+len(dd.withdrawList))
+	for _, p := range dd.newPathList {
+		l = append(l, p)
 	}
+	for _, p := range dd.withdrawList {
+		l = append(l, p.Clone(true))
+	}
+	return l
 }
 
-// Calculates best-path among known paths for this destination.
-//
-// Modifies destination's state related to stored paths. Removes withdrawn
-// paths from known paths. Also, adds new paths to known paths.
-func (dest *Destination) Calculate(ids []string, peerDown bool) (map[string]*Path, map[string]*Path, []*Path) {
-	bestList := make(map[string]*Path, len(ids))
-	oldList := make(map[string]*Path, len(ids))
-	oldKnownPathList := dest.knownPathList
-	// First remove the withdrawn paths.
-	dest.explicitWithdraw()
-	// Do implicit withdrawal
-	dest.implicitWithdraw()
-	// Collect all new paths into known paths.
-	dest.knownPathList = append(dest.knownPathList, dest.newPathList...)
-	// Clear new paths as we copied them.
-	dest.newPathList = make([]*Path, 0)
-	// Compute new best path
-	dest.computeKnownBestPath()
-
-	f := func(id string) (*Path, *Path) {
-		old := getBestPath(id, &oldKnownPathList)
-		best := dest.GetBestPath(id)
+func (dd *Destination) GetChanges(id string, peerDown bool) (*Path, *Path, []*Path) {
+	best, old := func(id string) (*Path, *Path) {
+		old := getBestPath(id, &dd.oldKnownPathList)
+		best := dd.GetBestPath(id)
 		if best != nil && best.Equal(old) {
 			// RFC4684 3.2. Intra-AS VPN Route Distribution
 			// When processing RT membership NLRIs received from internal iBGP
@@ -300,35 +278,99 @@ func (dest *Destination) Calculate(ids []string, peerDown bool) (map[string]*Pat
 			return old.Clone(true), old
 		}
 		return best, old
-	}
+	}(id)
 
 	var multi []*Path
-	for _, id := range ids {
-		bestList[id], oldList[id] = f(id)
-		if id == GLOBAL_RIB_NAME && UseMultiplePaths.Enabled {
-			diff := func(lhs, rhs []*Path) bool {
-				if len(lhs) != len(rhs) {
+
+	if id == GLOBAL_RIB_NAME && UseMultiplePaths.Enabled {
+		diff := func(lhs, rhs []*Path) bool {
+			if len(lhs) != len(rhs) {
+				return true
+			}
+			for idx, l := range lhs {
+				if !l.Equal(rhs[idx]) {
 					return true
 				}
-				for idx, l := range lhs {
-					if !l.Equal(rhs[idx]) {
-						return true
-					}
-				}
-				return false
 			}
-			oldM := getMultiBestPath(id, &oldKnownPathList)
-			newM := dest.GetMultiBestPath(id)
-			if diff(oldM, newM) {
-				multi = newM
-				if len(newM) == 0 {
-					multi = []*Path{bestList[id]}
-				}
+			return false
+		}
+		oldM := getMultiBestPath(id, &dd.oldKnownPathList)
+		newM := dd.GetMultiBestPath(id)
+		if diff(oldM, newM) {
+			multi = newM
+			if len(newM) == 0 {
+				multi = []*Path{best}
 			}
 		}
 	}
+	return best, old, multi
+}
 
-	return bestList, oldList, multi
+func (dd *Destination) AddWithdraw(withdraw *Path) {
+	dd.validatePath(withdraw)
+	dd.withdrawList = append(dd.withdrawList, withdraw)
+}
+
+func (dd *Destination) AddNewPath(newPath *Path) {
+	dd.validatePath(newPath)
+	dd.newPathList = append(dd.newPathList, newPath)
+}
+
+func (dd *Destination) validatePath(path *Path) {
+	if path == nil || path.GetRouteFamily() != dd.routeFamily {
+
+		log.WithFields(log.Fields{
+			"Topic":      "Table",
+			"Key":        dd.GetNlri().String(),
+			"Path":       path,
+			"ExpectedRF": dd.routeFamily,
+		}).Error("path is nil or invalid route family")
+	}
+}
+
+// Calculates best-path among known paths for this destination.
+//
+// Modifies destination's state related to stored paths. Removes withdrawn
+// paths from known paths. Also, adds new paths to known paths.
+func (dest *Destination) Calculate() *Destination {
+	oldKnownPathList := dest.knownPathList
+	newPathList := dest.newPathList
+	// First remove the withdrawn paths.
+	withdrawn := dest.explicitWithdraw()
+	// Do implicit withdrawal
+	dest.implicitWithdraw()
+
+	for _, path := range withdrawn {
+		if id := path.GetNlri().PathLocalIdentifier(); id != 0 {
+			dest.localIdMap.Unflag(uint(id))
+		}
+	}
+	// Collect all new paths into known paths.
+	dest.knownPathList = append(dest.knownPathList, dest.newPathList...)
+
+	for _, path := range dest.knownPathList {
+		if path.GetNlri().PathLocalIdentifier() == 0 {
+			id, err := dest.localIdMap.FindandSetZeroBit()
+			if err != nil {
+				dest.localIdMap.Expand()
+				id, _ = dest.localIdMap.FindandSetZeroBit()
+			}
+			path.GetNlri().SetPathLocalIdentifier(uint32(id))
+		}
+	}
+	// Clear new paths as we copied them.
+	dest.newPathList = make([]*Path, 0)
+	// Compute new best path
+	dest.computeKnownBestPath()
+
+	return &Destination{
+		routeFamily:      dest.routeFamily,
+		nlri:             dest.nlri,
+		knownPathList:    dest.knownPathList,
+		oldKnownPathList: oldKnownPathList,
+		newPathList:      newPathList,
+		withdrawList:     withdrawn,
+	}
 }
 
 // Removes withdrawn paths.
@@ -373,14 +415,15 @@ func (dest *Destination) explicitWithdraw() paths {
 	for _, withdraw := range dest.withdrawList {
 		isFound := false
 		for _, path := range dest.knownPathList {
-			// We have a match if the source are same.
-			if path.GetSource().Equal(withdraw.GetSource()) {
+			// We have a match if the source and path-id are same.
+			if path.GetSource().Equal(withdraw.GetSource()) && path.GetNlri().PathIdentifier() == withdraw.GetNlri().PathIdentifier() {
 				isFound = true
 				// this path is referenced in peer's adj-rib-in
 				// when there was no policy modification applied.
 				// we could flag IsWithdraw down after use to avoid
 				// a path with IsWithdraw flag exists in adj-rib-in
 				path.IsWithdraw = true
+				withdraw.GetNlri().SetPathLocalIdentifier(path.GetNlri().PathLocalIdentifier())
 				matches = append(matches, withdraw)
 			}
 		}
@@ -425,7 +468,7 @@ func (dest *Destination) implicitWithdraw() paths {
 			// version num. as newPaths are implicit withdrawal of old
 			// paths and when doing RouteRefresh (not EnhancedRouteRefresh)
 			// we get same paths again.
-			if newPath.GetSource().Equal(path.GetSource()) {
+			if newPath.GetSource().Equal(path.GetSource()) && newPath.GetNlri().PathIdentifier() == path.GetNlri().PathIdentifier() {
 				log.WithFields(log.Fields{
 					"Topic": "Table",
 					"Key":   dest.GetNlri().String(),
@@ -433,6 +476,7 @@ func (dest *Destination) implicitWithdraw() paths {
 				}).Debug("Implicit withdrawal of old path, since we have learned new path from the same peer")
 
 				found = true
+				newPath.GetNlri().SetPathLocalIdentifier(path.GetNlri().PathLocalIdentifier())
 				break
 			}
 		}
@@ -702,7 +746,9 @@ func compareByASPath(path1, path2 *Path) *Path {
 	attribute1 := path1.getPathAttr(bgp.BGP_ATTR_TYPE_AS_PATH)
 	attribute2 := path2.getPathAttr(bgp.BGP_ATTR_TYPE_AS_PATH)
 
-	if attribute1 == nil || attribute2 == nil {
+	// With addpath support, we could compare paths from API don't
+	// AS_PATH. No need to warn here.
+	if !path1.IsLocal() && !path2.IsLocal() && (attribute1 == nil || attribute2 == nil) {
 		log.WithFields(log.Fields{
 			"Topic":   "Table",
 			"Key":     "compareByASPath",
@@ -986,7 +1032,7 @@ func (old *Destination) Select(option ...DestinationSelectOption) *Destination {
 			}
 		}
 	}
-	new := NewDestination(old.nlri)
+	new := NewDestination(old.nlri, 0)
 	for _, path := range paths {
 		p := path.Clone(path.IsWithdraw)
 		p.Filter("", path.Filtered(id))
