@@ -63,7 +63,7 @@ func Test_InterfaceUpdateBody(t *testing.T) {
 	pos := INTERFACE_NAMSIZ
 	binary.BigEndian.PutUint32(buf[pos:], 1)
 	pos += 4
-	buf[pos] = INTERFACE_ACTIVE
+	buf[pos] = byte(INTERFACE_ACTIVE)
 	pos += 1
 	binary.BigEndian.PutUint64(buf[pos:], 1)
 	pos += 8 // flags
@@ -95,18 +95,21 @@ func Test_InterfaceAddressUpdateBody(t *testing.T) {
 	assert := assert.New(t)
 
 	//DecodeFromBytes
-	buf := make([]byte, 11)
+	buf := make([]byte, 15)
 	pos := 0
-	binary.BigEndian.PutUint32(buf[pos:], 0)
+	binary.BigEndian.PutUint32(buf[pos:], 0) // index
 	pos += 4
-	buf[pos] = 0x01
+	buf[pos] = 0x01 // flags
 	pos += 1
-	buf[pos] = 0x2
+	buf[pos] = 0x2 // family
 	pos += 1
-	ip := net.ParseIP("192.168.100.1").To4()
+	ip := net.ParseIP("192.168.100.1").To4() // prefix
 	copy(buf[pos:pos+4], []byte(ip))
 	pos += 4
-	buf[pos] = byte(24)
+	buf[pos] = byte(24) // prefix len
+	pos += 1
+	dst := net.ParseIP("192.168.100.255").To4() // destination
+	copy(buf[pos:pos+4], []byte(dst))
 
 	b := &InterfaceAddressUpdateBody{}
 	err := b.DecodeFromBytes(buf, 2)
@@ -114,6 +117,7 @@ func Test_InterfaceAddressUpdateBody(t *testing.T) {
 	assert.Equal(uint8(1), b.Flags)
 	assert.Equal("192.168.100.1", b.Prefix.String())
 	assert.Equal(uint8(24), b.Length)
+	assert.Equal("192.168.100.255", b.Destination.String())
 
 	// af invalid
 	buf[5] = 0x4
@@ -157,7 +161,7 @@ func Test_IPRouteBody_IPv4(t *testing.T) {
 	buf := make([]byte, 26)
 	buf[0] = byte(ROUTE_CONNECT)
 	buf[1] = byte(FLAG_SELECTED)
-	buf[2] = MESSAGE_NEXTHOP | MESSAGE_DISTANCE | MESSAGE_METRIC | MESSAGE_MTU
+	buf[2] = byte(MESSAGE_NEXTHOP | MESSAGE_DISTANCE | MESSAGE_METRIC | MESSAGE_MTU)
 	buf[3] = 24
 	ip := net.ParseIP("192.168.100.0").To4()
 	copy(buf[4:7], []byte(ip))
@@ -177,7 +181,7 @@ func Test_IPRouteBody_IPv4(t *testing.T) {
 	assert.Equal(nil, err)
 	assert.Equal("192.168.100.0", r.Prefix.String())
 	assert.Equal(uint8(0x18), r.PrefixLength)
-	assert.Equal(uint8(MESSAGE_NEXTHOP|MESSAGE_DISTANCE|MESSAGE_METRIC|MESSAGE_MTU), r.Message)
+	assert.Equal(MESSAGE_NEXTHOP|MESSAGE_DISTANCE|MESSAGE_METRIC|MESSAGE_MTU, r.Message)
 	assert.Equal("0.0.0.0", r.Nexthops[0].String())
 	assert.Equal(uint32(1), r.Ifindexs[0])
 	assert.Equal(uint8(0), r.Distance)
@@ -205,7 +209,7 @@ func Test_IPRouteBody_IPv4(t *testing.T) {
 	buf = make([]byte, 18)
 	buf[0] = byte(ROUTE_CONNECT)
 	buf[1] = byte(FLAG_SELECTED)
-	buf[2] = MESSAGE_NEXTHOP | MESSAGE_DISTANCE | MESSAGE_METRIC
+	buf[2] = byte(MESSAGE_NEXTHOP | MESSAGE_DISTANCE | MESSAGE_METRIC)
 	buf[3] = 24
 	ip = net.ParseIP("192.168.100.0").To4()
 	copy(buf[4:7], []byte(ip))
@@ -223,7 +227,7 @@ func Test_IPRouteBody_IPv4(t *testing.T) {
 	buf = make([]byte, 12)
 	buf[0] = byte(ROUTE_CONNECT)
 	buf[1] = byte(FLAG_SELECTED)
-	buf[2] = MESSAGE_DISTANCE | MESSAGE_METRIC
+	buf[2] = byte(MESSAGE_DISTANCE | MESSAGE_METRIC)
 	buf[3] = 24
 	ip = net.ParseIP("192.168.100.0").To4()
 	copy(buf[4:7], []byte(ip))
@@ -242,7 +246,7 @@ func Test_IPRouteBody_IPv6(t *testing.T) {
 	buf := make([]byte, 43)
 	buf[0] = byte(ROUTE_CONNECT)
 	buf[1] = byte(FLAG_SELECTED)
-	buf[2] = MESSAGE_NEXTHOP | MESSAGE_DISTANCE | MESSAGE_METRIC | MESSAGE_MTU
+	buf[2] = byte(MESSAGE_NEXTHOP | MESSAGE_DISTANCE | MESSAGE_METRIC | MESSAGE_MTU)
 	buf[3] = 64
 	ip := net.ParseIP("2001:db8:0:f101::").To16()
 	copy(buf[4:12], []byte(ip))
@@ -263,7 +267,7 @@ func Test_IPRouteBody_IPv6(t *testing.T) {
 	assert.Equal(nil, err)
 	assert.Equal("2001:db8:0:f101::", r.Prefix.String())
 	assert.Equal(uint8(64), r.PrefixLength)
-	assert.Equal(uint8(MESSAGE_NEXTHOP|MESSAGE_DISTANCE|MESSAGE_METRIC|MESSAGE_MTU), r.Message)
+	assert.Equal(MESSAGE_NEXTHOP|MESSAGE_DISTANCE|MESSAGE_METRIC|MESSAGE_MTU, r.Message)
 	assert.Equal("::", r.Nexthops[0].String())
 	assert.Equal(uint32(1), r.Ifindexs[0])
 	assert.Equal(uint8(0), r.Distance)
@@ -298,7 +302,7 @@ func Test_IPRouteBody_IPv6(t *testing.T) {
 	buf = make([]byte, 50)
 	buf[0] = byte(ROUTE_CONNECT)
 	buf[1] = byte(FLAG_SELECTED)
-	buf[2] = MESSAGE_NEXTHOP | MESSAGE_DISTANCE | MESSAGE_METRIC
+	buf[2] = byte(MESSAGE_NEXTHOP | MESSAGE_DISTANCE | MESSAGE_METRIC)
 	buf[3] = 24
 	ip = net.ParseIP("2001:db8:0:f101::").To4()
 	copy(buf[4:12], []byte(ip))
@@ -316,7 +320,7 @@ func Test_IPRouteBody_IPv6(t *testing.T) {
 	buf = make([]byte, 11)
 	buf[0] = byte(ROUTE_CONNECT)
 	buf[1] = byte(FLAG_SELECTED)
-	buf[2] = MESSAGE_DISTANCE | MESSAGE_METRIC
+	buf[2] = byte(MESSAGE_DISTANCE | MESSAGE_METRIC)
 	buf[3] = 16
 	ip = net.ParseIP("2501::").To16()
 	copy(buf[4:6], []byte(ip))
